-- k.yokomizo
library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.std_logic_unsigned.all;
    
entity CTRL is
port(
        CLK    : in std_logic;
        RESETB  : in std_logic;
        START    : in std_logic;
        ADJUST   : in std_logic;
        LAP      : in std_logic;
        Q_B      : in std_logic_vector(3 downto 0);
        EN_A    : in std_logic;
        CNT_EN   : out std_logic;
        DATA_SEL : out std_logic;
        WREN     : out std_logic;
        ADR   : out std_logic_vector(4 downto 0);
        LAP_NO   : out std_logic_vector(4 downto 0)
        );
end CTRL;

Architecture RTL of CTRL is
    signal STATE : std_logic_vector(1 downto 0);
    signal LAP_STOP  : std_logic;
    signal WR_ADR: std_logic_vector(4 downto 0);
    signal LAP_ADR: std_logic_vector(4 downto 0);
    signal TMP_TIM: std_logic_vector(3 downto 0);
    signal DATA_SEL_SET  : std_logic;
    constant HI_LEVEL : std_logic := '1';
    constant IDLE:std_logic_vector(1 downto 0):="00";
    constant COUNT:std_logic_vector(1 downto 0):="01";
    constant LAP_VIEW:std_logic_vector(1 downto 0):="10";
begin
--
--
--

process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      STATE <= IDLE ;
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      case STATE is
        when IDLE =>
          if START='1' then
            STATE <= COUNT;
          elsif LAP='1' then
            STATE <= LAP_VIEW;
          else
            STATE <= IDLE;
          end if;
        when COUNT =>
          if START='1' then
            STATE <= IDLE;
          else
            STATE <= COUNT;
          end if;   
        when LAP_VIEW =>
          if START='1' then
            STATE <= COUNT;
          elsif ADJUST='1' then
            STATE <= IDLE;
          else
            STATE <= LAP_VIEW;
          end if;
        when others =>
          STATE <= IDLE;
      end case;  
   end if;
end process;

CNT_EN <= '1' when STATE = COUNT else '0';

LAP_STOP <=  '1' when ((LAP='1')or((START='1')and(STATE=COUNT))) else '0';
  
process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      WR_ADR <= "00000";
      LAP_ADR <= "00000";
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if ADJUST='1'  then
        WR_ADR <= "00000";
        LAP_ADR <= "00000";
      elsif (LAP_STOP='1') then
        if  STATE = COUNT then
           WR_ADR <= WR_ADR + "00001";
           LAP_ADR <= LAP_ADR + "00001";
        else
          WR_ADR <= WR_ADR;
          if LAP_ADR = WR_ADR then
             LAP_ADR <= "00000" ;
          else
             LAP_ADR <= LAP_ADR + "00001";
          end if;
        end if;
      end if;
   end if;
end process;

WREN <= '1' when (LAP_STOP='1')and(STATE=COUNT) else '0';

process(CLK,RESETB)
begin
   if(RESETB = '0') then  
     ADR <= "00000";
   elsif(CLK'event and CLK = '1')  then   -- CLK event
     if (STATE=COUNT) then 
        ADR <= WR_ADR ;
     else 
        ADR <= LAP_ADR;
     end if;
   end if;
end process;

LAP_NO <= LAP_ADR;

process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      DATA_SEL <= '0';
      TMP_TIM  <= "0000";
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if (ADJUST='1')or(START='1')  then
        DATA_SEL <= '0';
        TMP_TIM <= "0000";
      elsif (STATE = COUNT) then
        if (DATA_SEL_SET='1')and(EN_A='1') then
           DATA_SEL <= '1';
           TMP_TIM <= Q_B;
        elsif (TMP_TIM=Q_B)and(EN_A='1') then
           DATA_SEL <='0';
        end if;
      elsif LAP='1' then
        DATA_SEL <= '1';
      end if;
   end if;
end process;

process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      DATA_SEL_SET <= '0';
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if (ADJUST='1')or(START='1')  then
        DATA_SEL_SET <= '0';
      elsif LAP='1' then
        DATA_SEL_SET <= '1';
      elsif EN_A='1' then
        DATA_SEL_SET <= '0';
      else
        DATA_SEL_SET <= DATA_SEL_SET ;
      end if;
   end if;
end process;

end RTL;






