-- k.yokomizo
library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.std_logic_unsigned.all;
    
entity OUTPUT_IF is
generic(
        LED_CYCLE : std_logic_vector(7 downto 0) :=  "11111111");        
port(
        CLK    : in std_logic;
        RESETB : in std_logic;
        Q_A    : in std_logic_vector(3 downto 0);
        Q_B    : in std_logic_vector(3 downto 0);
        Q_C    : in std_logic_vector(3 downto 0);
        Q_D    : in std_logic_vector(3 downto 0);
        LAP_NO : in std_logic_vector(4 downto 0);
        WEN    : in std_logic;
        CNT_EN : in std_logic;
        MEM_DATA: in std_logic_vector(15 downto 0);
        DATA_SEL  : in std_logic;
        SEG_OUT     : out std_logic_vector(7 downto 0);
        SEG_SEL     : out std_logic_vector(5 downto 0)
        );
end OUTPUT_IF;

Architecture RTL of OUTPUT_IF is
    signal LAP_NUM_A : std_logic_vector(3 downto 0) ;
    signal LAP_NUM_B : std_logic_vector(3 downto 0) ;
    signal SEG_DATA  : std_logic_vector(3 downto 0) ;
    signal SEG_DATA0  : std_logic_vector(3 downto 0) ;
    signal SEG_DATA1  : std_logic_vector(3 downto 0) ;
    signal SEG_DATA2  : std_logic_vector(3 downto 0) ;
    signal SEG_DATA3  : std_logic_vector(3 downto 0) ;
    signal SEG_DATA4  : std_logic_vector(3 downto 0) ;
    signal SEG_DATA5  : std_logic_vector(3 downto 0) ;
    signal LED_CNT_A : std_logic_vector(7 downto 0);
    signal LED_CNT_B : std_logic_vector(2 downto 0);
    signal LAP_DATA : std_logic_vector(15 downto 0);  
    constant HI_LEVEL : std_logic := '1';
    --constant LED_CYCLE : std_logic_vector := "11111111";
    --constant LED_CYCLE : std_logic_vector :=  "00000001";  --test
begin
--
--
--
 
process(CLK,RESETB)
begin
   if(RESETB = '0') then 
      LAP_DATA <= "0000000000000000";
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if WEN='1' then
        LAP_DATA <= Q_D & Q_C & Q_B & Q_A;
      end if;
   end if;
end process;



process(DATA_SEL,CNT_EN,MEM_DATA,LAP_DATA,Q_A,Q_B,Q_C,Q_D)
begin
   if DATA_SEL='0' then
      SEG_DATA0 <= Q_A; 
      SEG_DATA1 <= Q_B; 
      SEG_DATA2 <= Q_C; 
      SEG_DATA3 <= Q_D;
   elsif CNT_EN='1' then
      SEG_DATA0 <= LAP_DATA( 3 downto 0); 
      SEG_DATA1 <= LAP_DATA( 7 downto 4); 
      SEG_DATA2 <= LAP_DATA(11 downto 8); 
      SEG_DATA3 <= LAP_DATA(15 downto 12); 
   else  
      SEG_DATA0 <= MEM_DATA( 3 downto 0); 
      SEG_DATA1 <= MEM_DATA( 7 downto 4); 
      SEG_DATA2 <= MEM_DATA(11 downto 8); 
      SEG_DATA3 <= MEM_DATA(15 downto 12); 
   end if;
end process;

SEG_DATA4 <= "0000" when DATA_SEL='0' else LAP_NUM_A ;
SEG_DATA5 <= "0000" when DATA_SEL='0' else LAP_NUM_B ;

LAP_NUM_A <= LAP_NO( 3 downto 0) when LAP_NO(3 downto 0) < "1010" else
             LAP_NO( 3 downto 0)-"1010" ;-- "1010";

LAP_NUM_B <= "0001" when LAP_NO(3 downto 0) > "1001" else "0000";

process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      LED_CNT_A <= "00000000" ;
      LED_CNT_B <= "000" ;
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if LED_CNT_A=LED_CYCLE  then    
         LED_CNT_A <="00000000";
         if LED_CNT_B="101" then
           LED_CNT_B <= "000";
         else
           LED_CNT_B <= LED_CNT_B + "001";
         end if;
      else
         LED_CNT_A <= LED_CNT_A + "00000001"; -- count up
         LED_CNT_B <= LED_CNT_B; 
      end if;
   end if;
end process;

process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      SEG_DATA <= "0000" ;
      SEG_SEL <= "111111" ;
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      case LED_CNT_B is
         when "000" =>
            SEG_DATA <= SEG_DATA0;
            SEG_SEL <="111110";
         when "001" =>
            SEG_DATA <= SEG_DATA1;
            SEG_SEL <="111101";
         when "010" =>
            SEG_DATA <= SEG_DATA2;
            SEG_SEL <="111011";
         when "011" =>
            SEG_DATA <= SEG_DATA3;
            SEG_SEL <="110111";     
         when "100" =>
            SEG_DATA <= SEG_DATA4;
            SEG_SEL <="101111";     
         when "101" =>
            SEG_DATA <= SEG_DATA5;
            SEG_SEL <="011111";         
         when others => null;
      end case;
   end if;
end process;

process(SEG_DATA)
  begin
      case SEG_DATA is
         when "0000" => SEG_OUT <= "11111100";
         when "0001" => SEG_OUT <= "01100000";
         when "0010" => SEG_OUT <= "11011010";
         when "0011" => SEG_OUT <= "11110010";
         when "0100" => SEG_OUT <= "01100110";
         when "0101" => SEG_OUT <= "10110110";
         when "0110" => SEG_OUT <= "10111110";
         when "0111" => SEG_OUT <= "11100100";
         when "1000" => SEG_OUT <= "11111110"; --8
         when "1001" => SEG_OUT <= "11100110"; --9
         when "1010" => SEG_OUT <= "11101110"; --A
         when "1011" => SEG_OUT <= "00111110"; --b
         when "1100" => SEG_OUT <= "10011100"; --c
         when "1101" => SEG_OUT <= "01111010"; --d
         when "1110" => SEG_OUT <= "01011110"; --e
         when "1111" => SEG_OUT <= "10001110"; --f
         when others => SEG_OUT <= "00000000";
      end case;
  end process;


end RTL;





