-- k.yokomizo
library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.std_logic_unsigned.all;
    
entity SW_IF is
port(
        CLK    : in std_logic;
        RESETB : in std_logic;
        START_SW  : in std_logic;
        ADJUST_SW  : in std_logic;
        LAP_SW  : in std_logic;
        START  : out std_logic;
        ADJUST : out std_logic;
        LAP    : out std_logic
        );
end SW_IF;

Architecture RTL of SW_IF is
    signal  START_CHK  : std_logic_vector(3 downto 0);
    signal  START_DFF      : std_logic;
    signal  ADJUST_CHK  : std_logic_vector(3 downto 0);
    signal  ADJUST_DFF      : std_logic;
    signal  LAP_CHK  : std_logic_vector(3 downto 0);
    signal  LAP_DFF      : std_logic;
    signal  SAMPLE_CNT  : std_logic_vector(3 downto 0);
    signal  SAMPLE_TIM     : std_logic;

begin
--
--
--
process(CLK,RESETB)
begin
   if(RESETB = '0') then
      SAMPLE_CNT <= "0000";
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      SAMPLE_CNT <= SAMPLE_CNT + "0001";
   end if;
end process;

SAMPLE_TIM <= '1' when SAMPLE_CNT="1111" else '0';
     

process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      START_CHK <= "1111" ;
      ADJUST_CHK <= "1111" ;
      LAP_CHK <= "1111" ;
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if SAMPLE_TIM='1' then
         START_CHK  <= START_CHK(2 downto 0) & START_SW;
         ADJUST_CHK <= ADJUST_CHK(2 downto 0) & ADJUST_SW;
         LAP_CHK    <= LAP_CHK(2 downto 0) & LAP_SW;
      end if;
   end if;
end process;
  
process(CLK,RESETB)
begin
   if(RESETB = '0') then                   -- Reset
      START_DFF <= '0' ;
      ADJUST_DFF <= '0' ;
      LAP_DFF <= '0' ;
   elsif(CLK'event and CLK = '1')  then   -- CLK event
      if (START_CHK="1100") then
        START_DFF <= '1';
      else
        START_DFF <= '0';
      end if;
      if (ADJUST_CHK="1100") then        
        ADJUST_DFF <= '1';
      else
        ADJUST_DFF <= '0';
      end if;
      if (LAP_CHK="1100")  then
        LAP_DFF <= '1';
      else
        LAP_DFF <= '0';
      end if;
   end if;
end process;

START <= '1' when (START_CHK="1100") and (START_DFF='0') else '0' ; 
ADJUST <= '1' when (ADJUST_CHK="1100") and (ADJUST_DFF='0') else '0' ; 
LAP <= '1' when (LAP_CHK="1100") and (LAP_DFF='0') else '0' ; 


end RTL;





