-- k.yokomizo
library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.std_logic_unsigned.all;
    
entity S_WATCH is
generic(
        BASE_MAX_C : std_logic_vector( 17 downto 0) := "101100111111111111";
        LED_CYCLE : std_logic_vector(7 downto 0) :=  "11111111");
port(
        CLK    : in std_logic;
        RESETB  : in std_logic;
        START_SW    : in std_logic;
        ADJUST_SW   : in std_logic;
        LAP_SW      : in std_logic;
        SEG_OUT     : out std_logic_vector(7 downto 0);
        SEG_SEL     : out std_logic_vector(5 downto 0);
        MON_0 :out std_logic;
        MON_1 :out std_logic
        );
end S_WATCH;

Architecture RTL of S_WATCH is
    signal  START      : std_logic;
    signal  ADJUST      : std_logic;
    signal  LAP      : std_logic;
    signal  Q_A      : std_logic_vector(3 downto 0) ;
    signal  Q_B      : std_logic_vector(3 downto 0) ;
    signal  Q_C      : std_logic_vector(3 downto 0) ;
    signal  Q_D      : std_logic_vector(3 downto 0) ;
    signal  EN_A    : std_logic;
    signal  CNT_EN    : std_logic;
    signal WREN    : std_logic;
    signal WR_DATA : std_logic_vector(15 downto 0) ;
    signal MEM_DATA : std_logic_vector(15 downto 0) ;
    signal ADR : std_logic_vector(4 downto 0) ;
    signal LAP_NO : std_logic_vector(4 downto 0) ;
    signal DATA_SEL    : std_logic;
   
    component  SW_IF 
        port(
        CLK    : in std_logic;
        RESETB : in std_logic;
        START_SW  : in std_logic;
        ADJUST_SW  : in std_logic;
        LAP_SW  : in std_logic;
        START  : out std_logic;
        ADJUST : out std_logic;
        LAP    : out std_logic
        );
    end component ;

    component CTRL 
        port(
        CLK    : in std_logic;
        RESETB  : in std_logic;
        START    : in std_logic;
        ADJUST   : in std_logic;
        LAP      : in std_logic;
        Q_B      : in std_logic_vector(3 downto 0);
        EN_A     : in std_logic;
        CNT_EN   : out std_logic;
        DATA_SEL : out std_logic;
        WREN     : out std_logic;
        ADR   : out std_logic_vector(4 downto 0);
        LAP_NO   : out std_logic_vector(4 downto 0)
        );
    end component ;
    
    component  TIMER 
        generic(
               BASE_MAX_C : std_logic_vector( 17 downto 0));
        port(
               CLK     : in std_logic;
               RESETB  : in std_logic;
               ADJUST  : in std_logic;
               CNT_EN  : in std_logic;
               Q_A     : out std_logic_vector(3 downto 0);
               Q_B     : out std_logic_vector(3 downto 0);
               Q_C     : out std_logic_vector(3 downto 0);
               Q_D     : out std_logic_vector(3 downto 0);
               EN_A    : out std_logic;
               MON_0   : out std_logic
        );
    end component ;
    
    component OUTPUT_IF
        generic(
        LED_CYCLE : std_logic_vector(7 downto 0) );
        port(
        CLK    : in std_logic;
        RESETB : in std_logic;
        Q_A    : in std_logic_vector(3 downto 0);
        Q_B    : in std_logic_vector(3 downto 0);
        Q_C    : in std_logic_vector(3 downto 0);
        Q_D    : in std_logic_vector(3 downto 0);
        LAP_NO : in std_logic_vector(4 downto 0);
        WEN    : in std_logic;
        CNT_EN    : in std_logic;
        MEM_DATA: in std_logic_vector(15 downto 0);
        DATA_SEL  : in std_logic;
        SEG_OUT     : out std_logic_vector(7 downto 0);
        SEG_SEL     : out std_logic_vector(5 downto 0)
        );
    end component ;
    

    component DATA_RAM2
    PORT
	(
		address		: IN STD_LOGIC_VECTOR (4 DOWNTO 0);
		we		: IN STD_LOGIC  := '1';
		data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		q		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
    end component;

begin

WR_DATA <= ( Q_D & Q_C & Q_B & Q_A);
  
SW_IF_U : SW_IF
   port map (
      CLK    => CLK,
      RESETB => RESETB,
      START_SW => START_SW,
      ADJUST_SW  => ADJUST_SW,
      LAP_SW => LAP_SW,
      START  => START ,
      ADJUST => ADJUST ,
      LAP    => LAP );


CTRL_U : CTRL
   port map (
      CLK    => CLK ,
      RESETB => RESETB,
      START  => START,
      ADJUST => ADJUST,
      LAP    => LAP,
      Q_B    => Q_B,
      EN_A  => EN_A,
      CNT_EN => CNT_EN,
      DATA_SEL => DATA_SEL,
      WREN     => WREN,
      ADR   => ADR,
      LAP_NO  => LAP_NO
      );
      
TIMER_U : TIMER
   generic map(
      BASE_MAX_C => BASE_MAX_C)
   port map(
      CLK => CLK,
      RESETB=> RESETB,
      ADJUST=> ADJUST,
      CNT_EN => CNT_EN,
      Q_A => Q_A,
      Q_B => Q_B,
      Q_C => Q_C,
      Q_D => Q_D,
      EN_A  => EN_A,
      MON_0 => MON_0 );

OUTPUT_IF_U : OUTPUT_IF
   generic map(
      LED_CYCLE => LED_CYCLE)
   port map (
      CLK => CLK,
      RESETB=> RESETB,
      Q_A => Q_A,
      Q_B => Q_B,
      Q_C => Q_C,
      Q_D => Q_D,
      WEN => WREN,
      CNT_EN => CNT_EN,
      LAP_NO => LAP_NO ,
      MEM_DATA => MEM_DATA ,
      DATA_SEL => DATA_SEL ,
      SEG_OUT => SEG_OUT,
      SEG_SEL => SEG_SEL );

RAM1: DATA_RAM2
   PORT MAP (
      data => WR_DATA,
      address => ADR,
      we      => WREN ,
      q       => MEM_DATA 	);

MON_1 <= Q_A(3);

end RTL;












