-- k.yokomizo
library IEEE;
    use IEEE.std_logic_1164.all;
 --   use IEEE.std_logic_unsigned.all; 
 use std.textio.all; 
 use ieee.std_logic_textio.all; 
    
entity T_S_WATCH is
        generic(
          --BASE_MAX_C : std_logic_vector( 17 downto 0) := "000000000000001000";
          BASE_MAX_C : std_logic_vector( 17 downto 0) := "101100111111111111";
          LED_CYCLE : std_logic_vector(7 downto 0) :=  "00000001");
end T_S_WATCH;

Architecture RTL of T_S_WATCH is
    signal   CLK    : std_logic;
    signal   RESETB  : std_logic;
    signal   START_SW    : std_logic;
    signal   ADJUST_SW   : std_logic;
    signal   LAP_SW      : std_logic;
    signal   SEG_OUT     : std_logic_vector(7 downto 0);
    signal   SEG_SEL     : std_logic_vector(5 downto 0);
    signal   SEG_SEL_D     : std_logic_vector(5 downto 0);
    signal   SEG_DATA    : integer:=0 ;
    signal   SEG_DATA0    : integer:=0 ;
    signal   SEG_DATA1    : integer:=0 ;
    signal   SEG_DATA2    : integer:=0 ;
    signal   SEG_DATA3    : integer:=0 ;
    signal   SEG_DATA4    : integer:=0 ;
    signal   SEG_DATA5    : integer:=0 ;
    signal   SEG_DATA0_D  : integer:=0 ;
    signal   PAT_NO       : integer:=0;
    signal   START_SW_D    : std_logic;
    signal   MON_0    : std_logic;
    signal   MON_1    : std_logic;
    constant STEP_A : time := 54 ns ;
    constant STEP_B : time := 540 ns ;
    file outfile :TEXT open write_mode is "SIMOUT.txt";
    
    component S_WATCH
        generic(
          BASE_MAX_C : std_logic_vector( 17 downto 0); 
          LED_CYCLE : std_logic_vector(7 downto 0) );
	PORT
	(
          CLK    : in std_logic;
          RESETB  : in std_logic;
          START_SW    : in std_logic;
          ADJUST_SW   : in std_logic;
          LAP_SW      : in std_logic;
          SEG_OUT     : out std_logic_vector(7 downto 0);
          SEG_SEL     : out std_logic_vector(5 downto 0);
          MON_0       : out std_logic;
          MON_1       : out std_logic
	);
    end component;

    
begin

DUT: S_WATCH
  generic map (
             BASE_MAX_C  => BASE_MAX_C, 
             LED_CYCLE => LED_CYCLE )
  PORT MAP ( CLK => CLK,
		        RESETB=> RESETB,
                        START_SW=> START_SW,
                        ADJUST_SW=> ADJUST_SW,
                        LAP_SW=> LAP_SW,
		        SEG_OUT => SEG_OUT,
		        SEG_SEL => SEG_SEL,
                        MON_0 => MON_0,
                        MON_1 => MON_1);

--
--
--
process
begin
   CLK <= '0';
   wait for STEP_A/2 ;
   CLK <= '1';
   wait for STEP_A/2 ;
   PAT_NO <= PAT_NO + 1;
end process;

process
  procedure INITIALIZE is begin    
    RESETB <= '0';
    START_SW <='1';
    ADJUST_SW <='1';
    LAP_SW <='1';
    wait for STEP_A ;
    RESETB <= '1';
    wait for STEP_A*2 ;
  end INITIALIZE;

  procedure START(T0:IN integer;T1:IN integer) is begin 
    START_SW <='0';               
    wait for STEP_A*T0 ;
    START_SW <='1';
    wait for STEP_A*T1 ;
  end START;
     
begin
   INITIALIZE;
   START(1,20);
   START(50,200000);
   START(50,200);
   --* stop *********************
   START_SW <='0';               
   wait for STEP_B*20 ;
   START_SW <='1';
   wait for STEP_B*20 ;
   --* start ******************** 
   START_SW <='0';
   wait for STEP_B*20 ;
   START_SW <='1';
   wait for STEP_B*20 ;
   --* adjust********************
   ADJUST_SW <='0';
   wait for STEP_B*20 ;
   ADJUST_SW <='1';
   wait for STEP_B*20 ;
   --* stop *********************
   START_SW <='0';      
   wait for STEP_B*20 ;
   START_SW <='1';
   wait for STEP_B*20 ;
   --* adjust********************
   ADJUST_SW <='0';
   wait for STEP_B*20 ;
   ADJUST_SW <='1';
   wait for STEP_B*20 ;
   --* start ******************** 
   START_SW <='0'; 
   wait for STEP_B*20 ;
   START_SW <='1';                
   wait for STEP_B*20 ;
   --* lap1  ******************** 
   LAP_SW <='0';
   wait for STEP_B*20 ;
   LAP_SW <='1';             
   wait for STEP_B*200 ;
   --* lap2  ******************** 
   LAP_SW <='0';                    
   wait for STEP_B*20 ;
   LAP_SW <='1';
   wait for STEP_B*200 ;
   --* stop *********************
   START_SW <='0';    
   wait for STEP_B*20 ;
   START_SW <='1';              
   wait for STEP_B*20 ;
   --* lap1  ******************** 
   LAP_SW <='0';
   wait for STEP_B*20 ;
   LAP_SW <='1';             
   wait for STEP_B*20 ;
   --* lap2  ******************** 
   LAP_SW <='0'; 
   wait for STEP_B*20 ;
   LAP_SW <='1';  
   wait for STEP_B*20 ;
   --* lap3  ********************
   LAP_SW <='0';
   wait for STEP_B*20 ;
   LAP_SW <='1';
   --* adjust********************
   ADJUST_SW <='0';
   wait for STEP_B*20 ;
   ADJUST_SW <='1';
   wait for STEP_B*20 ;
   wait;
end process;

process(SEG_OUT)
  begin
    case SEG_OUT is
      when "11111100" => SEG_DATA <= 0;
      when "01100000" => SEG_DATA <= 1;
      when "11011010" => SEG_DATA <= 2;
      when "11110010" => SEG_DATA <= 3;
      when "01100110" => SEG_DATA <= 4;
      when "10110110" => SEG_DATA <= 5;
      when "10111110" => SEG_DATA <= 6;
      when "11100100" => SEG_DATA <= 7;
      when "11111110" => SEG_DATA <= 8; --//8
      when "11100110" => SEG_DATA <= 9; --//9
      when "11101110" => SEG_DATA <= 10; --//A
      when "00111110" => SEG_DATA <= 11; --//b
      when "10011100" => SEG_DATA <= 12; --//c
      when "01111010" => SEG_DATA <= 13; --//d
      when "01011110" => SEG_DATA <= 14; --//e
      when "10001110" => SEG_DATA <= 15; --//f
      when others     => SEG_DATA <= 0;
    end case ;
    end process ;

SEG_SEL_D <= SEG_SEL after 1 ns ;

process(SEG_SEL_D)
  begin
    if SEG_SEL="111110" then
      SEG_DATA0 <= SEG_DATA;
    end if;
    if SEG_SEL="111101" then
      SEG_DATA1 <= SEG_DATA;
    end if;
    if SEG_SEL="111011" then
      SEG_DATA2 <= SEG_DATA;
    end if;
    if SEG_SEL="110111" then
      SEG_DATA3 <= SEG_DATA;
    end if;
    if SEG_SEL="101111" then
      SEG_DATA4 <= SEG_DATA;
    end if;
    if SEG_SEL="011111" then
      SEG_DATA5 <= SEG_DATA;
    end if;
  end process ;

process(CLK)
   variable ln:line;
begin
  if(CLK'event and CLK = '1')  then   -- CLK event
    SEG_DATA0_D <= SEG_DATA0;
    START_SW_D <= START_SW ;
    if START_SW /= START_SW_D then
      write(ln,string'("NO:"));
      write(ln,PAT_NO);
      write(ln,string'("START_SW= "));
      write(ln,START_SW);
      writeline(outfile,ln);      
    end if;
    if SEG_DATA0_D /= SEG_DATA0 then
      write(ln,string'("NO:"));
      write(ln,PAT_NO);
      write(ln,string'(" LED= "));
      write(ln,(SEG_DATA5));
      write(ln,string'(" "));
      write(ln,(SEG_DATA4));
      write(ln,string'(" "));
      write(ln,(SEG_DATA3));
      write(ln,string'(" "));
      write(ln,(SEG_DATA2));
      write(ln,string'(" "));
      write(ln,(SEG_DATA1));
      write(ln,string'(" "));
      write(ln,(SEG_DATA0));
      writeline(outfile,ln);
    end if;
  end if;
end process;

end RTL;
